#ifndef FILTER_H
#define	FILTER_H

#include <sys/types.h>		/* IEEE Std 1003.1-2001 non-conformance. */
#include <regex.h>

#include "account.h"


/*
 * Mode for masks that comprise a filter. So if a filter is of AND type then
 * as default masks are ANDed.
 */
#define FILTER_MODE_OR			1
#define FILTER_MODE_AND			2

/* Action to do in case of filter matching. */
#define FILTER_ACTION_DELETE		1
#define FILTER_ACTION_COPY		2
#define FILTER_ACTION_MOVE		3
#define FILTER_ACTION_RCOPY		4
#define FILTER_ACTION_RMOVE		5
#define FILTER_ACTION_FLAG_REPLACE	6
#define FILTER_ACTION_FLAG_ADD		7
#define FILTER_ACTION_FLAG_REMOVE	8
#define FILTER_ACTION_LIST		9

/* Message flags to replace/add/remove. */
#define MESSAGE_FLAG_NONE		0x00
#define MESSAGE_FLAG_SEEN		0x01
#define MESSAGE_FLAG_ANSWERED		0x02
#define MESSAGE_FLAG_FLAGGED		0x04
#define MESSAGE_FLAG_DELETED		0x08
#define MESSAGE_FLAG_DRAFT		0x10

/* Type of mask. This overrides the default filter mode (AND/OR). */
#define MASK_TYPE_OR			1
#define MASK_TYPE_AND			2

/* Type of mask match according to the four different regular expressions. */
#define MASK_MATCH_1            	1
#define MASK_MATCH_2            	2
#define MASK_MATCH_3            	3
#define MASK_MATCH_4            	4

/* String length of action's arguments. */
#define ARGS_LEN			256

/* String length of mask's body. */
#define MASK_BODY_LEN			256

/* Maximum filters assigned to a mailbox. */
#define MBOX_FILTERS_MAX		64


/* Filter. */
typedef struct filter {
	char key[KEY_LEN];	/* Alias of filter. */
	unsigned int mode;	/* AND/OR mode. */

	struct {		/* What to do on filter match. */
		unsigned int type;	/* Action. */
		account_t *raccount;	/* Remote account to rcopy/rmove
					 * message. */
		char destmbox[MBOX_NAME_LEN];	/* Destination mailbox. */
		unsigned int msgflags;	/* Message flags to
					 * replace/add/remove. */
		char args[ARGS_LEN];	/* Action's arguments. */
	} action;

	struct mask *masks;	/* Masks comprising the filter. */
	unsigned int masknum;	/* Total number of masks. */
	unsigned int masklen;	/* Total length of body of masks. */

	struct filter *left, *right;	/* Left/right node of tree. */
} filter_t;

/* Mask. */
typedef struct mask {
	char body[MASK_BODY_LEN];	/* Body of mask. */
	unsigned int type;	/* AND/OR type. */
	struct mask *next;	/* Next node of linked list. */
} mask_t;


/*	filter.c	*/
int set_filter(char *line, regmatch_t * match);
int set_action(char *line, regmatch_t * match);
int set_mask(char *line, regmatch_t * match, int mmt);
int set_job(char *line, regmatch_t * match);


#endif				/* FILTER_H */
